@@ -33,6 +33,7 @@
 #include "notification-plugin.h"
 #include "notification-plugin-log.h"
 #include "notification-plugin-dialogs.h"
+#include "notification-plugin-settings-ui.h"
 
 /* prototypes */
 static void
@@ -452,43 +453,11 @@ notification_plugin_size_changed (XfcePanelPlugin       *plugin,
                                   gint                   size,
                                   NotificationPlugin    *notification_plugin)
 {
-#if !LIBXFCE4PANEL_CHECK_VERSION (4, 13, 0)
-  GtkStyleContext *context;
-  GtkBorder padding, border;
-  gint width;
-  gint xthickness;
-  gint ythickness;
-#endif
-  gint icon_size;
+  gint nrows = xfce_panel_plugin_get_nrows (notification_plugin->plugin);
 
-  size /= xfce_panel_plugin_get_nrows (notification_plugin->plugin);
-  gtk_widget_set_size_request (GTK_WIDGET (notification_plugin->button), size, size);
-#if LIBXFCE4PANEL_CHECK_VERSION (4,13,0)
-  icon_size = xfce_panel_plugin_get_icon_size (XFCE_PANEL_PLUGIN (plugin));
-#else
-  /* Calculate the size of the widget because the theme can override it */
-  context = gtk_widget_get_style_context (GTK_WIDGET (notification_plugin->button));
-  gtk_style_context_get_padding (context, gtk_widget_get_state_flags (GTK_WIDGET (notification_plugin->button)), &padding);
-  gtk_style_context_get_border (context, gtk_widget_get_state_flags (GTK_WIDGET (notification_plugin->button)), &border);
-  xthickness = padding.left + padding.right + border.left + border.right;
-  ythickness = padding.top + padding.bottom + border.top + border.bottom;
+  gtk_widget_set_size_request(GTK_WIDGET(notification_plugin->button), size / nrows, size / nrows);
 
-  /* Calculate the size of the space left for the icon */
-  width = size - 2 * MAX (xthickness, ythickness);
-
-  /* Since symbolic icons are usually only provided in 16px we
-   * try to be clever and use size steps */
-  if (width <= 21)
-    icon_size = 16;
-  else if (width >=22 && width <= 29)
-    icon_size = 24;
-  else if (width >= 30 && width <= 40)
-    icon_size = 32;
-  else
-    icon_size = width;
-#endif
-
-  notification_plugin->icon_size = icon_size;
+  notification_plugin->icon_size = xfce_panel_plugin_get_icon_size(plugin);
   notification_plugin_update_icon(notification_plugin);
 
   return TRUE;
@@ -504,6 +473,8 @@ notification_plugin_construct (XfcePanelPlugin *plugin)
   /* setup transation domain */
   xfce_textdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
 
+  notification_plugin_settings_ui_register_resource();
+
   /* create the plugin */
   notification_plugin = notification_plugin_new (plugin);
 
