@@ -42,9 +42,8 @@
 #include <common/xfce-notify-log-gbus.h>
 #include <common/xfce-notify-log-util.h>
 
+#include "xfce-notify-config-ui.h"
 #include "xfce-notify-log-viewer.h"
-#include "xfce4-notifyd-config.ui.h"
-#include "xfce4-notifyd-config-known-app.ui.h"
 
 typedef struct
 {
@@ -645,7 +644,7 @@ xfce4_notifyd_known_application_insert_row (SettingsPanel *panel,
     gtk_icon_size_lookup(GTK_ICON_SIZE_LARGE_TOOLBAR, &icon_width, &icon_height);
     icon_size = MIN(icon_width, icon_height);
 
-    builder = gtk_builder_new_from_string(xfce4_notifyd_config_known_app_ui, xfce4_notifyd_config_known_app_ui_length);
+    builder = gtk_builder_new_from_resource("/org/xfce/notifyd/settings/xfce4-notifyd-config-known-app.glade");
     if (G_UNLIKELY(builder == NULL)) {
         g_critical("Unable to load known app UI description");
         return;
@@ -974,6 +973,7 @@ xfce4_notifyd_config_setup_dialog(SettingsPanel *panel, GtkBuilder *builder) {
     GtkWidget *known_applications_scrolled_window;
     GtkWidget *known_applications_listbox;
     GtkWidget *placeholder_label;
+    GtkWidget *display_fields_combo;
     GtkWidget *do_not_disturb_switch;
     GtkWidget *do_not_disturb_info;
     GtkWidget *log_switch;
@@ -1005,6 +1005,13 @@ xfce4_notifyd_config_setup_dialog(SettingsPanel *panel, GtkBuilder *builder) {
         GENERAL   *
      **************/
     // Behavior
+    display_fields_combo = GTK_WIDGET(gtk_builder_get_object(builder, "display_fields"));
+    xfconf_g_property_bind(panel->channel, NOTIFICATION_DISPLAY_FIELDS_PROP, G_TYPE_STRING,
+                           G_OBJECT(display_fields_combo), "active-id");
+    if (gtk_combo_box_get_active_id(GTK_COMBO_BOX(display_fields_combo)) == NULL) {
+        gtk_combo_box_set_active_id(GTK_COMBO_BOX(display_fields_combo), DISPLAY_FIELDS_DEFAULT);
+    }
+
     do_not_disturb_switch = GTK_WIDGET (gtk_builder_get_object (builder, "do_not_disturb"));
     xfconf_g_property_bind (panel->channel, "/do-not-disturb", G_TYPE_BOOLEAN,
                             G_OBJECT (do_not_disturb_switch), "active");
@@ -1015,6 +1022,10 @@ xfce4_notifyd_config_setup_dialog(SettingsPanel *panel, GtkBuilder *builder) {
     g_signal_connect (G_OBJECT (do_not_disturb_switch), "state-set",
                       G_CALLBACK (xfce4_notifyd_do_not_disturb_activated), do_not_disturb_info);
 
+    btn = GTK_WIDGET(gtk_builder_get_object(builder, "gauge_ignores_dnd"));
+    xfconf_g_property_bind(panel->channel, GAUGE_IGNORES_DND_PROP, G_TYPE_BOOLEAN,
+                           G_OBJECT(btn), "active");
+
     primary_monitor = GTK_WIDGET(gtk_builder_get_object(builder, "primary_monitor"));
     xfconf_g_property_bind(panel->channel, "/primary-monitor", G_TYPE_UINT,
                            G_OBJECT(primary_monitor), "active");
@@ -1232,8 +1243,9 @@ main(int argc,
       g_warning("Failed to initialize libnotify.");
     }
 
-    builder = gtk_builder_new();
-    gtk_builder_add_from_string(builder, xfce4_notifyd_config_ui, xfce4_notifyd_config_ui_length, NULL);
+    xfce_notify_config_ui_register_resource();
+
+    builder = gtk_builder_new_from_resource("/org/xfce/notifyd/settings/xfce4-notifyd-config.glade");
     if(G_UNLIKELY(!builder)) {
         g_error("Unable to read embedded UI definition file");
         return EXIT_FAILURE;
