@@ -31,6 +31,11 @@
 
 #include <libxfce4ui/libxfce4ui.h>
 
+#ifdef ENABLE_X11
+#include <X11/Xlib.h>
+#include <gdk/gdkx.h>
+#endif
+
 #ifdef ENABLE_WAYLAND
 #include <gdk/gdkwayland.h>
 #include <gtk-layer-shell.h>
@@ -66,6 +71,7 @@ struct _XfceNotifyWindow
     gint last_monitor;
     gboolean override_redirect;
 
+    XfceNotifyUrgency urgency;
     guint expire_timeout;
 
     gboolean mouse_hover;
@@ -221,10 +227,12 @@ xfce_notify_window_init(XfceNotifyWindow *window)
     GdkMonitor *monitor;
     GdkRectangle geometry;
 
+    window->urgency = XFCE_NOTIFY_URGENCY_NORMAL;
     window->expire_timeout = DEFAULT_EXPIRE_TIMEOUT;
     window->normal_opacity = DEFAULT_NORMAL_OPACITY;
     window->do_fadeout = DEFAULT_DO_FADEOUT;
     window->do_slideout = DEFAULT_DO_SLIDEOUT;
+    window->last_monitor = -1;
 
 #ifdef ENABLE_WAYLAND
     if (GDK_IS_WAYLAND_DISPLAY(gdk_display_get_default())) {
@@ -343,7 +351,7 @@ xfce_notify_window_init(XfceNotifyWindow *window)
 static void
 xfce_notify_window_start_expiration(XfceNotifyWindow *window)
 {
-    if(window->expire_timeout) {
+    if (window->expire_timeout > 0 && window->urgency != XFCE_NOTIFY_URGENCY_CRITICAL) {
         gint64 ct;
         guint timeout;
         gboolean fade_transparent;
@@ -400,6 +408,46 @@ xfce_notify_window_realize(GtkWidget *widget)
 
     gdk_window_set_override_redirect(gtk_widget_get_window(widget), window->override_redirect);
 
+#ifdef ENABLE_X11
+    // GDK includes WM_TAKE_FOCUS in WM_PROTOCOLS, even for non-focusable
+    // windows, and (notably) xfwm4 will still try to give focus to the window,
+    // which will cause any fullscreen windows to un-fullscreen.  The ICCCM
+    // says that windows that do not take input should not advertise support
+    // for WM_TAKE_FOCUS, so I believe GDK is in the wrong here.  Even if it
+    // does make sense for the WM to handle this differently, our windows
+    // should follow the spec!
+    if (GDK_IS_X11_DISPLAY(gdk_display_get_default())) {
+        Display *dpy = gdk_x11_display_get_xdisplay(gdk_display_get_default());
+        Window win = gdk_x11_window_get_xid(gtk_widget_get_window(widget));
+        Atom *wm_protocols = NULL;
+        int n_protocols = 0;
+
+        gdk_x11_display_error_trap_push(gdk_display_get_default());
+
+        if (XGetWMProtocols(dpy, win, &wm_protocols, &n_protocols) != 0 && wm_protocols != NULL) {
+            Atom wm_take_focus = XInternAtom(dpy, "WM_TAKE_FOCUS", False);
+            gboolean found = FALSE;
+
+            for (int i = 0; i < n_protocols; ++i) {
+                if (wm_protocols[i] == wm_take_focus) {
+                    found = TRUE;
+                }
+                if (found && i + 1 < n_protocols) {
+                    wm_protocols[i] = wm_protocols[i + 1];
+                }
+            }
+
+            if (found) {
+                XSetWMProtocols(dpy, win, wm_protocols, n_protocols - 1);
+            }
+
+            XFree(wm_protocols);
+        }
+
+        gdk_x11_display_error_trap_pop_ignored(gdk_display_get_default());
+    }
+#endif
+
     xfce_notify_window_start_expiration(window);
 
 #ifdef ENABLE_SOUND
@@ -970,6 +1018,42 @@ xfce_notify_window_set_expire_timeout(XfceNotifyWindow *window,
     }
 }
 
+void
+xfce_notify_window_set_urgency(XfceNotifyWindow *window,
+                               XfceNotifyUrgency urgency)
+{
+    g_return_if_fail(XFCE_IS_NOTIFY_WINDOW(window));
+
+    if (window->urgency != urgency) {
+        if (window->urgency == XFCE_NOTIFY_URGENCY_CRITICAL) {
+            // No longer critical, so start expiration
+            if (window->expire_id == 0 && window->fade_id == 0 && gtk_widget_get_realized(GTK_WIDGET(window))) {
+                xfce_notify_window_start_expiration(window);
+            }
+        }
+
+        window->urgency = urgency;
+
+        if (window->urgency == XFCE_NOTIFY_URGENCY_CRITICAL) {
+            if (window->fade_id != 0) {
+                g_source_remove(window->fade_id);
+                window->fade_id = 0;
+            }
+            if (window->expire_id != 0) {
+                g_source_remove(window->expire_id);
+                window->expire_id = 0;
+            }
+            gtk_widget_set_opacity(GTK_WIDGET(window), window->normal_opacity);
+        }
+    }
+}
+
+XfceNotifyUrgency
+xfce_notify_window_get_urgency(XfceNotifyWindow *window) {
+    g_return_val_if_fail(XFCE_IS_NOTIFY_WINDOW(window), XFCE_NOTIFY_URGENCY_NORMAL);
+    return window->urgency;
+}
+
 void
 xfce_notify_window_set_actions(XfceNotifyWindow *window,
                                const gchar **actions,
@@ -998,18 +1082,25 @@ xfce_notify_window_set_actions(XfceNotifyWindow *window,
     for(i = 0; actions && actions[i]; i += 2) {
         const gchar *cur_action_id = actions[i];
         const gchar *cur_button_text = actions[i+1];
+        const gchar *icon_name = NULL;
         GtkWidget *btn, *img = NULL, *lbl;
         gdouble padding;
 
         if(!cur_button_text || !cur_action_id || !*cur_action_id)
             break;
-        /* Gnome applications seem to send a "default" action which often has no
-           label or text, because it is intended to be executed when clicking
-           the notification window.
-           See https://developer.gnome.org/notification-spec/
-           As we do not support this for the moment we hide buttons without labels. */
-        if (g_strcmp0 (cur_button_text, "") == 0)
-            continue;
+
+        if (actions_are_icon_names) {
+            icon_name = cur_action_id;
+        }
+
+        if (cur_button_text == NULL || g_strcmp0 (cur_button_text, "") == 0) {
+            // Actions with no label are intended to be the 'default' action; the spec
+            // suggests that clicking the notification will activate the default action,
+            // but we just close them on click, so let's create a button for it.
+            if (icon_name == NULL) {
+                icon_name = "emblem-default-symbolic";
+            }
+        }
 
         gtk_widget_style_get(GTK_WIDGET(window),
                              "padding", &padding,
@@ -1025,16 +1116,23 @@ xfce_notify_window_set_actions(XfceNotifyWindow *window,
                          G_CALLBACK(xfce_notify_window_button_clicked),
                          window);
 
-        if (actions_are_icon_names && gtk_icon_theme_has_icon(icon_theme, cur_action_id)) {
-            GIcon *icon = g_themed_icon_new_with_default_fallbacks(cur_action_id);
+        if (icon_name != NULL && gtk_icon_theme_has_icon(icon_theme, icon_name)) {
+            GIcon *icon = g_themed_icon_new_with_default_fallbacks(icon_name);
+            const gchar *desc;
+
+            if (cur_button_text == NULL || g_strcmp0 (cur_button_text, "") == 0) {
+                desc = _("Default Action");
+            } else {
+                desc = cur_button_text;
+            }
 
             img = gtk_image_new_from_gicon(icon, GTK_ICON_SIZE_BUTTON);
             gtk_widget_set_tooltip_text(img, cur_button_text);
-            atk_object_set_description(gtk_widget_get_accessible(img), cur_button_text);
+            atk_object_set_description(gtk_widget_get_accessible(img), desc);
             gtk_widget_show(img);
             gtk_container_add(GTK_CONTAINER(btn), img);
 
-            gtk_widget_set_tooltip_text(btn, cur_button_text);
+            gtk_widget_set_tooltip_text(btn, desc);
 
             g_object_unref(icon);
         }
